MODULE geometry
  USE field_mod
  
  TYPE t_geometry
    TYPE(t_field),POINTER :: xyz_i(:)
    TYPE(t_field),POINTER :: xyz_e(:)
    TYPE(t_field),POINTER :: xyz_v(:)
    TYPE(t_field),POINTER :: Ai(:)
    TYPE(t_field),POINTER :: Av(:)
    TYPE(t_field),POINTER :: de(:)
    TYPE(t_field),POINTER :: le(:)
    TYPE(t_field),POINTER :: Riv(:)
    TYPE(t_field),POINTER :: Riv2(:)
    TYPE(t_field),POINTER :: ne(:)
    TYPE(t_field),POINTER :: Wee(:)
    TYPE(t_field),POINTER :: bi(:)
    TYPE(t_field),POINTER :: fv(:)
  END TYPE t_geometry    

  TYPE(t_geometry),TARGET :: geom
  
  REAL(rstd),POINTER :: Ai(:)
  REAL(rstd),POINTER :: xyz_i(:,:)
  REAL(rstd),POINTER :: xyz_e(:,:)
  REAL(rstd),POINTER :: xyz_v(:,:)
  REAL(rstd),POINTER :: Av(:)
  REAL(rstd),POINTER :: de(:)
  REAL(rstd),POINTER :: le(:)
  REAL(rstd),POINTER :: Riv(:,:)
  REAL(rstd),POINTER :: Riv2(:,:)
  INTEGER,POINTER :: ne(:,:)
  REAL(rstd),POINTER :: Wee(:,:,:)
  REAL(rstd),POINTER :: bi(:)
  REAL(rstd),POINTER :: fv(:)
     


CONTAINS

  SUBROUTINE allocate_geometry
  USE field_mod
  IMPLICIT NONE
  
    CALL allocate_field(geom%Ai,field_t,type_real)
    CALL allocate_field(geom%xyz_i,field_t,type_real,3)
    CALL allocate_field(geom%xyz_e,field_u,type_real,3)
    CALL allocate_field(geom%xyz_v,field_z,type_real,3)
    CALL allocate_field(geom%de,field_u,type_real)
    CALL allocate_field(geom%le,field_u,type_real)
    CALL allocate_field(geom%bi,field_t,type_real)
    CALL allocate_field(geom%Av,field_z,type_real)
    CALL allocate_field(geom%Riv,field_t,type_real,6)
    CALL allocate_field(geom%Riv2,field_t,type_real,6)
    CALL allocate_field(geom%ne,field_t,type_integer,6)
    CALL allocate_field(geom%Wee,field_u,type_real,5,2)
    CALL allocate_field(geom%bi,field_t,type_real)
    CALL allocate_field(geom%fv,field_z,type_real)

  END SUBROUTINE allocate_geometry
  
  
  SUBROUTINE swap_geometry(ind)
  USE field_mod
  IMPLICIT NONE
    INTEGER,INTENT(IN) :: ind
    Ai=geom%Ai(ind)
    xyz_i=geom%xyz_i(ind)
    xyz_e=geom%xyz_e(ind)
    xyz_v=geom%xyz_v(ind)
    de=geom%de(ind)
    le=geom%le(ind)
    Av=geom%Av(ind)
    Riv=geom%Riv(ind)
    Riv2=geom%Riv2(ind)
    ne=geom%ne(ind)
    Wee=geom%Wee(ind)
    bi=geom%bi(ind)
    fv=geom%fv(ind)
    
  END SUBROUTINE swap_geometry
    
  SUBROUTINE set_geometry
  USE metric
  USE spherical_geom_mod
  USE domain_mod
  USE dimensions
  IMPLICIT NONE

    REAL(rstd) :: surf(6)
    REAL(rstd) :: surf_v(6)
    INTEGER :: ind,i,j,k,n
    TYPE(t_domain),POINTER :: d
    REAL(rstd) ::  S
    REAL(rstd) :: w(6)
    REAl(rstd) :: lon,lat
    INTEGER :: ii_glo,jj_glo
    REAL(rstd) :: S1,S2
      
    DO ind=1,ndomain
      d=>domain(ind)
      CALL swap_dimensions(ind)
      CALL swap_geometry(ind)
      DO j=jj_begin-1,jj_end+1
        DO i=ii_begin-1,ii_end+1
          n=(j-1)*iim+i
        
          xyz_i(n,:)=d%xyz(:,i,j) 
          xyz_v(n+z_up,:)=d%vertex(:,vup-1,i,j) 
          xyz_v(n+z_down,:)=d%vertex(:,vdown-1,i,j) 
          
          CALL xyz2lonlat(xyz_v(n+z_up,:),lon,lat)
          fv(n+z_up)=2*sin(lat)*omega
          CALL xyz2lonlat(xyz_v(n+z_down,:),lon,lat)
          fv(n+z_down)=2*sin(lat)*omega
          
          bi(n)=0. 
        
          CALL dist_cart(d%xyz(:,i,j),d%neighbour(:,right-1,i,j),de(n+u_right))
          CALL dist_cart(d%xyz(:,i,j),d%neighbour(:,lup-1,i,j),de(n+u_lup))
          CALL dist_cart(d%xyz(:,i,j),d%neighbour(:,ldown-1,i,j),de(n+u_ldown))
          
          CALL div_arc_bis(d%xyz(:,i,j),d%neighbour(:,right-1,i,j),0.5,xyz_e(n+u_right,:))
          CALL div_arc_bis(d%xyz(:,i,j),d%neighbour(:,lup-1,i,j),0.5,xyz_e(n+u_lup,:))
          CALL div_arc_bis(d%xyz(:,i,j),d%neighbour(:,ldown-1,i,j),0.5,xyz_e(n+u_ldown,:))
          
          CALL dist_cart(d%vertex(:,vrdown-1,i,j),d%vertex(:,vrup-1,i,j),le(n+u_right))
          CALL dist_cart(d%vertex(:,vup-1,i,j),d%vertex(:,vlup-1,i,j),le(n+u_lup))
          CALL dist_cart(d%vertex(:,vldown-1,i,j),d%vertex(:,vdown-1,i,j),le(n+u_ldown))

         
          Ai(n)=0
          DO k=0,5
            CALL surf_triangle(d%xyz(:,i,j),d%neighbour(:,k,i,j),d%neighbour(:,MOD((k+1+6),6),i,j),surf_v(k+1))
            CALL surf_triangle(d%xyz(:,i,j),d%vertex(:,MOD((k-1+6),6),i,j),d%vertex(:,k,i,j),surf(k+1))
            ne(n,k+1)=d%ne(k,i,j)
            Ai(n)=Ai(n)+surf(k+1)
          ENDDO


          DO k=0,5
            CALL surf_triangle(d%xyz(:,i,j),d%vertex(:,k,i,j),d%neighbour(:,k,i,j),S1)
            CALL surf_triangle(d%xyz(:,i,j),d%vertex(:,k,i,j),d%neighbour(:,MOD(k+1+6,6),i,j),S2)
            Riv(n,k+1)=0.5*(S1+S2)/Ai(n)
            Riv2(n,k+1)=0.5*(S1+S2)/surf_v(k+1)
          ENDDO

          DO k=1,6
            IF (ABS(surf_v(k))<1e-30) THEN
              Riv(n,k)=0.
            ENDIF
          ENDDO
          
          Av(n+z_up)=surf_v(vup)+1e-100
          Av(n+z_down)=surf_v(vdown)+1e-100
        
        ENDDO
      ENDDO
      
      DO j=jj_begin,jj_end
        DO i=ii_begin,ii_end
          n=(j-1)*iim+i
   
          CALL compute_wee(n,right,w)
          Wee(n+u_right,:,1)=w(1:5)

          CALL compute_wee(n+t_right,left,w)
          Wee(n+u_right,:,2)=w(1:5)


          CALL compute_wee(n,lup,w)
          Wee(n+u_lup,:,1)=w(1:5)

          CALL compute_wee(n+t_lup,rdown,w)
          Wee(n+u_lup,:,2)=w(1:5)


          CALL compute_wee(n,ldown,w)
          Wee(n+u_ldown,:,1)=w(1:5)

          CALL compute_wee(n+t_ldown,rup,w)
          Wee(n+u_ldown,:,2)=w(1:5)

        ENDDO
      ENDDO
      
      DO j=jj_begin,jj_end
        DO i=ii_begin,ii_end
          n=(j-1)*iim+i
          ii_glo=d%ii_begin_glo-d%ii_begin+i
          jj_glo=d%jj_begin_glo-d%jj_begin+j
          
          IF (ii_glo==1 .AND. jj_glo==1) THEN
            le(n+u_ldown)=0
            xyz_v(n+z_ldown,:)=xyz_v(n+z_down,:)
                       
          ENDIF

          IF (ii_glo==iim_glo .AND. jj_glo==1) THEN
            le(n+u_right)=0
            xyz_v(n+z_rdown,:)=xyz_v(n+z_rup,:)
          ENDIF

          IF (ii_glo==iim_glo .AND. jj_glo==jjm_glo) THEN
            le(n+u_rup)=0
            xyz_v(n+z_rup,:)=xyz_v(n+z_up,:)
          ENDIF

          IF (ii_glo==1 .AND. jj_glo==jjm_glo) THEN
            le(n+u_lup)=0
            xyz_v(n+z_up,:)=xyz_v(n+z_lup,:)
          ENDIF
          
        ENDDO
      ENDDO

      DO j=jj_begin-1,jj_end+1
        DO i=ii_begin-1,ii_end+1
          n=(j-1)*iim+i
          xyz_i(n,:)=xyz_i(n,:) * radius
          xyz_v(n+z_up,:)=xyz_v(n+z_up,:) * radius
          xyz_v(n+z_down,:)=xyz_v(n+z_down,:) *radius
          de(n+u_right)=de(n+u_right) * radius
          de(n+u_lup)=de(n+u_lup)*radius
          de(n+u_ldown)=de(n+u_ldown)*radius
          xyz_e(n+u_right,:)=xyz_e(n+u_right,:)*radius
          xyz_e(n+u_lup,:)=xyz_e(n+u_lup,:)*radius
          xyz_e(n+u_ldown,:)=xyz_e(n+u_ldown,:)*radius
          le(n+u_right)=le(n+u_right)*radius
          le(n+u_lup)=le(n+u_lup)*radius
          le(n+u_ldown)=le(n+u_ldown)*radius
          Ai(n)=Ai(n)*radius**2
          Av(n+z_up)=Av(n+z_up)*radius**2
          Av(n+z_down)=Av(n+z_down)*radius**2
        ENDDO
      ENDDO
                 
    ENDDO
    CALL surf_triangle(d%xyz(:,ii_begin,jj_begin),d%xyz(:,ii_begin,jj_end),d%xyz(:,ii_end,jj_begin),S)
!    PRINT *,"Surf triangle : ",S*20/(4*Pi)
  
  END SUBROUTINE set_geometry
  
  SUBROUTINE compute_wee(n,pos,w)
  IMPLICIT NONE
    INTEGER,INTENT(IN) :: n
    INTEGER,INTENT(IN) :: pos
    REAL(rstd),INTENT(OUT) ::w(6)

    REAL(rstd) :: ne_(0:5)
    REAL(rstd) :: Riv_(6)
    INTEGER    :: k
    
    
    DO k=0,5
      ne_(k)=ne(n,MOD(pos-1+k+6,6)+1)  
      Riv_(k+1)=Riv(n,MOD(pos-1+k+6,6)+1)
    ENDDO
          
    w(1)=-ne_(0)*ne_(1)*(Riv_(1)-0.5)
    w(2)=-ne_(2)*(ne_(0)*Riv_(2)-w(1)*ne_(1))
    w(3)=-ne_(3)*(ne_(0)*Riv_(3)-w(2)*ne_(2))
    w(4)=-ne_(4)*(ne_(0)*Riv_(4)-w(3)*ne_(3))
    w(5)=-ne_(5)*(ne_(0)*Riv_(5)-w(4)*ne_(4))
    w(6)=ne_(0)*ne_(5)*(Riv_(6)-0.5)
    
!    IF ( ABS(w(5)-w(6))>1e-20) PRINT *, "pb pour wee : w(5)!=w(6)",sum(Riv_(:))

   END SUBROUTINE compute_wee
            

  
  SUBROUTINE compute_geometry
  IMPLICIT NONE
    CALL allocate_geometry
    CALL set_geometry
    
  END SUBROUTINE compute_geometry
  
END MODULE geometry
